timerlow = $00
timerhi = $01

nmi_count = $03

temp0 = $10
temp1 = $11
temp2 = $12
temp3 = $13
waitdelay = $18

gloveData = $20
gloveByte0 = $20
gloveByte1 = $21
gloveX = $21
gloveByte2 = $22
gloveY = $22
gloveByte3 = $23
gloveZ = $23
gloveByte4 = $24
gloveRot = $24
gloveByte5 = $25
gloveFingers = $25
gloveByte6 = $26
gloveButtons = $26
gloveByte7 = $27
gloveByte8 = $28
gloveByte9 = $29
gloveByteA = $2A
gloveByteB = $2B
; reserved until $40, but hopefully only ised through $31


gloveConfig = $40
; $40-$46 reserved
gloveConfigIndex = $47
frameDelay = $48 ; keep the value of this >=2
pollBytes = $49 ; cap at 17 for sanity, if >12 only first 12 are displayed

gloveFirstQuery = $4D

pad2Old = $4E
pad2Buttons = $4F

nmi_acache = $FD
nmi_xcache = $FE
nmi_ycache = $FF

	.inesprg 1 ; code, footer
	.ineschr 1 ; not even used- bg, sprites
	.inesmir 0 ; not even used
	.inesmap 0 ; not even used, but may want to use 4 (MMC3) eventually


	.bank 0
	.org $8000	; first bank, top of the code

nmi:
	; cache regs
 sta nmi_acache
 stx nmi_xcache
 sty nmi_ycache
 lda $2002	; clear vblank interrupt
 lda #$00
 sta $2000	; reset vblank interrupt generation
 lda #$80	
 sta $2000	; re-set vblank interrupt generation
 
 inc timerlow		; increase the timer every frame
 bne nmi_no_inchi	; if timer lobyte is 0
 inc timerhi			; then inc the hibyte
nmi_no_inchi:

 inc nmi_count
	; uncache regs
 lda nmi_acache
 ldx nmi_xcache
 ldy nmi_ycache

 rti

irq:
 rti
 
;  ----==== Subroutines ====----
 
; --++ waitloop ++--
    ; waits a number of 1/60 second NMI-clocked frames, then returns
waitloop:
 lda waitdelay	; load the number of frames to wait
 clc
 adc timerlow	; calculate an exit time
 waitloop_innerloop:
 cmp timerlow
 bne waitloop_innerloop	; loop until we reach the exit time
 rts
; --++ end subroutine ++--
 
; --++ waitNMI ++-- 
    ; waits for a NMI, then returns
waitNMI:
 lda #0
 sta nmi_count
waitNMI_loop
 cmp nmi_count
 beq waitNMI_loop
 rts
; --++ end subroutine ++--

; --++ waitGloveClock ++-- 
    ; Mein Timingfheurer says clocks need to come at 22microsecond intervals
    ; or, per calculations below, 39.38 cycles.  Call it 39 for convenience.
    ; lda [absolute] takes 4 cycles, so pad with 35 cycles
    ; since we'll be doing this oh-so-many times, make it a routine.  jsr/rts soak cycles nicely anyway.
    ; after jsr and rts, we only need 22 cycles.  2 10s and a 2.
    ; REVISION: clock waiting does not appear to be necessary
waitGloveClock:
    ; jsr soaks 6 cycles
 ldx #2 ; 2 cycles
waitGloveClockDelay:
 iny ; 2 cycles, dummy op
 dey ; 2 cycles, dummy op
 dex ; 2 cycles
 bne waitGloveClockDelay ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
 ldx #0 ; 2 cycles to absorb the lack of branch
 
 rts ; 7 cycles
; --++ end subroutine ++--
  
; --++ waitGloveByte ++--
    ; Mein Timingfheurer also says bytes need to be spaced by 96microseconds
    ; or, per calculations below, 171.84 cycles.  Call it 173 for convenience.
    ; since we'll be doing this oh-so-many times, make it a routine.  jsr/rts soak cycles nicely anyway.
    ; after jsr and rts, we need 160 cycles.  16 10s.
waitGloveByte:
    ; jsr soaks 6 cycles
 ldx #16 ; 2 cycles
waitGloveByteDelay:
 iny ; 2 cycles, dummy op
 dey ; 2 cycles, dummy op
 dex ; 2 cycles
 bne waitGloveByteDelay ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
 ; usual wait-2 here is soaked by the initial ldx.
 
 rts ; 7 cycles
; --++ end subroutine ++--
 
; --++ flashGlove ++--
    ; One callable routine to write the gloveConfig values to the glove whenever we want
    ; If you want your x/y/a/temp data after this call, back it up yourself
flashGlove:
 
;glovePing:
    ; per The Infallable Trace Diagram, at startup
    ; knock knock
    ; REVISION: this is not required, but may still help detect the presence of the glove
; lda #1
; sta $4016
; lda #0
; sta $4016
 
; lda $4016
; jsr waitGloveClock
; lda $4016
; jsr waitGloveClock
; lda $4016
; jsr waitGloveClock
; lda $4016
; jsr waitGloveClock
    ; who's there?  I don't care; I declare it to be a Power Glove
    ; the actual return bits suggesting this to be a Power Glove are left as an exercise to posterity.
    
gloveFlag:
    ; we "need" to wait 7212 microseconds
    ; at 1.79MHz, 1 cycle is about .559 microseconds
    ; meaning we need to burn 12909.48 cycles.  Call it 12910.
    ; so construct a quick 10-cycle loop and run it 1291 times.  
    ; Or how abut 1290, since there are a few other extraneous instructions being thrown about?
    ; REVISION: this is not required, but I'm leaving the comment-code as reference
    
; ldx #0 ; 1-byte immediate = 2 cycles load->store EXTRA
 
;gloveFlagDelay1: ; 10-cycle 256-loop
; iny ; 2 cycles, dummy op
; iny ; 2 cycles, dummy op
; dex ; 2 cycles
; bne gloveFlagDelay1 ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
; iny ; 2 cycle dummy to absorb the 2 cycles of not-branching
 
;gloveFlagDelay2: ; 10-cycle 256-loop
; iny ; 2 cycles, dummy op
; iny ; 2 cycles, dummy op
; dex ; 2 cycles
; bne gloveFlagDelay2 ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
; iny ; 2 cycle dummy to absorb the 2 cycles of not-branching
 
;gloveFlagDelay3: ; 10-cycle 256-loop
; iny ; 2 cycles, dummy op
; iny ; 2 cycles, dummy op
; dex ; 2 cycles
; bne gloveFlagDelay3 ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
; iny ; 2 cycle dummy to absorb the 2 cycles of not-branching
 
;gloveFlagDelay4: ; 10-cycle 256-loop
; iny ; 2 cycles, dummy op
; iny ; 2 cycles, dummy op
; dex ; 2 cycles
; bne gloveFlagDelay4 ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
; iny ; 2 cycle dummy to absorb the 2 cycles of not-branching
 
;gloveFlagDelay5: ; 10-cycle 256-loop
; iny ; 2 cycles, dummy op
; iny ; 2 cycles, dummy op
; dex ; 2 cycles
; bne gloveFlagDelay5 ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
; iny ; 2 cycle dummy to absorb the 2 cycles of not-branching
 
    ; at this point we have only 10 more loops to go, so
; ldx #10 ; 2 cycles EXTRA
 
;gloveFlagDelay6: ; 10-cycle 10-loop
; iny ; 2 cycles, dummy op
; iny ; 2 cycles, dummy op
; dex ; 2 cycles
; bne gloveFlagDelay6 ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
; iny ; 2 cycle dummy to absorb the 2 cycles of not-branching
 
gloveInit:
    ; now that we've done some stuff for 7212 microseconds or close-e-dang-nuff
    ; start a nice wide latch pulse to let the Glove know config data is coming
    ; this IS almost certainly needed, and I won't bother to pin down the precise length
    ; because 2260micro seems to work just fine
 lda #1
 sta $4016 
 
    ; we "need" to wait 2260 microseconds
    ; at 1.79MHz, 1 cycle is about .559 microseconds --== NOTE: PAL SYSTEMS RUN AT 1.66! ==--
    ; meaning we need to burn 4045.4 cycles.  Call it 4045.
    ; so construct a quick 10-cycle loop and run it 404.5 times.  
    ; Or how abut 404, since there are a few other extraneous instructions being thrown about?
    
 ldx #0 ; 1-byte immediate = 2 cycles load->store EXTRA
 
gloveInitDelay1: 
 iny ; 2 cycles, dummy op
 iny ; 2 cycles, dummy op
 dex ; 2 cycles
 bne gloveInitDelay1 ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
 iny ; 2 cycle dummy to absorb the 2 cycles of not-branching
 
    ; only need 148 more, so
 ldx #148 ; 2 cycles EXTRA
 
gloveInitDelay2: 
 iny ; 2 cycles, dummy op
 iny ; 2 cycles, dummy op
 dex ; 2 cycles
 bne gloveInitDelay2 ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
 iny ; 2 cycle dummy to absorb the 2 cycles of not-branching
 
    ; now that we've done some more stuff for "2260" microseconds 
    
    ; "yo, Glove, listen up"
 lda #0
 sta $4016
    ; pad a few cycles, maybe not the full 5micro, but give it some time
    ; besides, this will make switching the latch faster
    ldx #0
    ldy #1
 
    ; 0x06, 0xC1, 0x08, 0x00, 0x02, 0xFF, 0x01 - a magic spell?
    
    
 ldx #0 ; byte index
gloveInitLoopByte:
 ldy #0 ; bit counter
    
 lda gloveConfig,x
 sta temp0 ; temp shift slot
 
gloveInitLoopBit:
    ; get the next init bit
 asl temp0
 ; rol A
 .db $2A ; stupid nesasm
 and #%00000001
    ; write the next init bit
 sta $4016
    ; clock through ("send") the init bit
 lda $4016
 
 iny
 cpy #8
 bne gloveInitLoopBit
 
 stx temp0 ; waitGloveByte makes x 0, and we don't need temp0 for now
 jsr waitGloveByte
 ldx temp0
 
 inx
 cpx #7
 bne gloveInitLoopByte
 
 
    ; 892 microseconds is 1596.68 cycles
    ; that's ~like~ 1600, or 160 10-loops
 ldx #160 ; 2 cycles EXTRA
 
gloveClearDelay: 
 iny ; 2 cycles, dummy op
 iny ; 2 cycles, dummy op
 dex ; 2 cycles
 bne gloveClearDelay ; branch takes 2 cycles to load/compare, and an additinal 2 if the branch is taken
 iny ; 2 cycle dummy to absorb the 2 cycles of not-branching
 
    ; aaaaand clear.  Finally.
 stx $4016
 
 lda #3
 sta waitdelay
    ; go make a sandwich (for something like 40000 microseconds.  Honestly, if it cares at this point, I will be sad.)
 jsr waitloop
 
 rts
; --++ end subroutine ++-- 

;  ----==== And the real code begins here ====----

start:

init:
	; set up the timer
 lda #$00
 sta timerhi
 lda #$00
 sta timerlow
 
 lda #0
 sta $2000 ; init PPU to name table 0, pattern tables 0, etc, w/o enabling NMI
 sta $2001 ; "turn off" PPU for massive BG load
 lda #$20
 sta $2006
 lda #$00
 sta $2006 ; set PPU address to $2000.  For once 16-bit addresses go is hibyte-first
 
 lda #$F0
 ldx #0
init_bg_blank_loop1: ; write 256 F0s to name table
 sta $2007
 inx
 bne init_bg_blank_loop1
init_bg_blank_loop2: ; and again- $2100-21FF
 sta $2007
 inx
 bne init_bg_blank_loop2
init_bg_blank_loop3: ; $2200-22ff
 sta $2007
 inx
 bne init_bg_blank_loop3
init_bg_blank_loop4: ; finally $2300-23ff
 sta $2007
 inx
 bne init_bg_blank_loop4
 

	; set initial palette
 lda #$3F
 sta $2006
 lda #$00
 sta $2006
 
 ldx #4;
paletteInit:
 lda #$0D
 sta $2007
 lda #$30
 sta $2007
 lda #$30
 sta $2007
 lda #$30
 sta $2007
 dex
 bne paletteInit
 
 ; sprites get 4 different palettes for illustration
 
 lda #$0D
 sta $2007
 lda #$30
 sta $2007
 sta $2007
 sta $2007
 
 lda #$0D
 sta $2007
 lda #$16
 sta $2007
 sta $2007
 sta $2007
 
 lda #$0D
 sta $2007
 lda #$2A
 sta $2007
 sta $2007
 sta $2007
 
 lda #$0D
 sta $2007
 lda #$12
 sta $2007
 sta $2007
 sta $2007
 
 
     ; assign palettes

 lda #$23
 sta $2006
 lda #$C0
 sta $2006

 ldx #$40
 lda #0
init_attributes:
 sta $2007
 dex
 bne init_attributes
 
 lda #0 ; proper way to end the update: zero the PPU address
 sta $2006
 sta $2006
 
 
    ; clear sprites
 lda #0
 sta $2003
 ldx #64
spriteInitLoop:
 lda #240 ; y pos
 sta $2004
 lda #0 ; tile
 sta $2004
 lda #%00000000 ; vhb???cc - [v]ert flip, [h]oriz flip, [b]ehind bg, [c]olor palette
 sta $2004
 lda #0 ; x pos
 sta $2004
 dex
 bne spriteInitLoop

 lda #%00011010
 sta $2001 ; init PPU to no-monochrome, bg in left-8, no sprites in left-8, bg on, sprites on, no color emphasis
 
	; enable NMI on vblank
 lda #$80
 sta $2000 ; NMI on, 0, 8x8 sprite, bg pattern table 0, sprite pattern table 0, increment by 1 (horiz), nametable 00
 
 lda #$1F	; 000 1 1 1 1 1 - enable all channels initially
 sta $4015
 
 
    ; per The Infallable Trace Diagram, at startup
    ; knock knock
    ; REVISION: this is not required, but may still help detect the presence of the glove.
    ; I'm not seeing any data back other than 0, though (may just be that I can't start up 
    ; without going thorugh the flashcart menu...)
 lda #1
 sta $4016
 lda #0
 sta $4016
 
 lda #0
 sta gloveFirstQuery
 lda $4016
 lsr A
 ; rol gloveFirstQuery
 .db $26, gloveFirstQuery ; stupid nesasm
 lda $4016
 lsr A
 ; rol gloveFirstQuery
 .db $26, gloveFirstQuery ; stupid nesasm
 lda $4016
 lsr A
 ; rol gloveFirstQuery
 .db $26, gloveFirstQuery ; stupid nesasm
 lda $4016
 lsr A
 ; rol gloveFirstQuery
 .db $26, gloveFirstQuery ; stupid nesasm
    ; who's there?  I don't care; I declare it to be a Power Glove
    ; but the return bits have been captured
 
    ; frame break and move on to time-conscious execution
    
 jsr waitNMI
 
initText:
    ; do some more cosmetic setup
 lda #$20
 sta $2006
 lda #$21
 sta $2006 ; set PPU address
 
 ldx #0
write_title:
 lda title,X
 cmp #$FF
 beq title_nowrite
 sta $2007
title_nowrite:
 inx
 cmp #$FF
 bne write_title
 
 lda #$20
 sta $2006
 lda #$41
 sta $2006 ; set PPU address
 
 ldx #0
write_copyright:
 lda copyright,X
 cmp #$FF
 beq copyright_nowrite
 sta $2007
copyright_nowrite:
 inx
 cmp #$FF
 bne write_copyright
 
 lda #$20
 sta $2006
 lda #$61
 sta $2006 ; set PPU address
 
 ldx #0
write_date:
 lda date,X
 cmp #$FF
 beq date_nowrite
 sta $2007
date_nowrite:
 inx
 cmp #$FF
 bne write_date
 
writeFirstReturn:
 lda #$20
 sta $2006
 lda #$99
 sta $2006 ; set PPU address
 lda gloveFirstQuery
 sta $2007

 lda #0 ; proper way to end the update: zero the PPU address
 sta $2006
 sta $2006
 
 ldx #0
 lda #$06 ; ?= header for 6 bytes of data
 sta gloveConfig,x
 inx
 lda #$C1 ; = ping rate C, mode 1
 sta gloveConfig,x
 inx
 lda #$08
 sta gloveConfig,x
 inx
 lda #$00
 sta gloveConfig,x
 inx
 lda #$02 ; = no apparent effect
 sta gloveConfig,x
 inx
 lda #$FF ; = field mask (1st 8; glove data in mode 1)
 sta gloveConfig,x
 inx
 lda #$01 ; = field mask (2nd 8; frame data in mode 1)
 sta gloveConfig,x
 
 lda #0
 sta gloveConfigIndex
 
 lda #4
 sta frameDelay;
 
 lda #12
 sta pollBytes
 

 jsr flashGlove
 
 jmp main_loop


;  ----==== Main Loop ====----

main_loop: 
 
 ; just poll the glove and display the bits
 
 jsr waitNMI
 
    ; draw the last 4 glove bytes (we can only update ~9 of these lines per frame)
 lda #$21
 sta $2006
 sta temp0
 lda #$A1
 sta $2006 ; set PPU address
 sta temp1
 
 ldx #8 ; byte index
bitsUpdate1ByteLoop:
 ldy #8 ; bit counter
 lda gloveData,X
bitsUpdate1BitLoop:
 sta temp2
 ; rol a
 .db $2A ; stupid nesasm
 ; rol a
 .db $2A ; stupid nesasm
 and #%00000001
 sta $2007
 lda temp2
 asl A
 dey
 bne bitsUpdate1BitLoop
 
 lda temp1
 clc
 adc #$20
 sta temp1
 cmp #$01
 bne bitsUpdate1SkipAddrHi
 inc temp0
bitsUpdate1SkipAddrHi:
 lda temp0
 sta $2006
 lda temp1
 sta $2006
 inx
 cpx #12
 bne bitsUpdate1ByteLoop
    
    ; also draw the current config
 lda #$23
 sta $2006
 lda #$01
 sta $2006 ; set PPU address
 
 ldx #0
configUpdateLoop:
 lda gloveConfig,x
 lsr A
 lsr A
 lsr A
 lsr A
 and #$0F
 sta $2007
 lda gloveConfig,x
 and #$0F
 sta $2007
 lda #$F0
 sta $2007
 inx
 cpx #7
 bne configUpdateLoop
 
    ; and the frame delay, which is really probably part of the config
 lda #$F0
 sta $2007
 sta $2007
 lda frameDelay
 sta $2007
 
    ; and the bytes per frame, which is really probably part of the config
 lda #$F0
 sta $2007
 sta $2007
 sta $2007
 lda pollBytes
 lsr A
 lsr A
 lsr A
 lsr A
 and #$0F
 sta $2007
 lda pollBytes
 and #$0F
 sta $2007
 

 ; config marker sprite
 lda #$F4
 sta $2003
 lda #183 ; y-1
 sta $2004
 lda #$22 ; tile
 sta $2004
 lda #%00000010 ; vhb???cc - [v]ert flip, [h]oriz flip, [b]ehind bg, [c]olor palette
 sta $2004
 lda #$0C ; x
 ldx gloveConfigIndex
 beq configPtrWrite
configPtrLoop:
 clc
 adc #24
 dex
 bne configPtrLoop
configPtrWrite:
 sta $2004
 
 ; frame/size marker sprite
 ;lda #$F8
 ;sta $2003
 lda #183 ; y-1
 sta $2004
 lda #$22 ; tile
 sta $2004
 lda #%00000001 ; vhb???cc - [v]ert flip, [h]oriz flip, [b]ehind bg, [c]olor palette
 sta $2004
 lda #192 ; x
 sta temp0
 lda pad2Buttons
 and #%00100000 ; select
 beq timingPtrWrite
 lda #36
 clc
 adc temp0
 sta temp0
timingPtrWrite:
 lda temp0
 sta $2004
 lda #0
 sta $2003 ; maybe I need to reset the sprite address too?  I shouldn't be getting mouse-trails on the NES...

 lda #0 ; proper way to end the update: zero the PPU address
 sta $2006
 sta $2006
 
    ; poll the glove
 
 ldx #0 ; byte index
glovePollLoopByte:
 ldy #0 ; bit counter

 lda #1
 sta $4016
 lda #0
 sta $4016
 
glovePollLoopBit:
 lda $4016
 eor #$FF ; inverts the data
 lsr A
 ; rol gloveData,X
 .db $36,gloveData ; stupid nesasm
 iny
 cpy #8
 bne glovePollLoopBit
 
 stx temp0 ; waitGloveByte makes x 0
 jsr waitGloveByte
 ldx temp0
 
 inx
 cpx pollBytes
 bne glovePollLoopByte
 
    ; grab controller 2 input to allow config adjustment
    ; %ABsSudlr
 lda pad2Buttons
 sta pad2Old
 lda #1
 sta $4017
 lda #0
 sta $4017
 ldy #8 ; bit counter
pad2PollLoopBit:
 lda $4017
 lsr A
 ; rol pad2Buttons
 .db $26,pad2Buttons ; stupid nesasm
 dey
 bne pad2PollLoopBit
 
    ; left/right (on press) changes target
 lda gloveConfigIndex
 cmp #6
 beq noIncIndex
 lda #%00000001
 and pad2Buttons
 beq noIncIndex
 lda #%00000001
 and pad2Old
 bne noIncIndex
 inc gloveConfigIndex
noIncIndex:
 
 lda gloveConfigIndex
 beq noDecIndex
 lda #%00000010
 and pad2Buttons
 beq noDecIndex
 lda #%00000010
 and pad2Old
 bne noDecIndex
 dec gloveConfigIndex
noDecIndex:

    ; up/down (on hold) changes value
 lda #%00001000
 and pad2Buttons
 beq noIncValue
 ldx gloveConfigIndex
 inc gloveConfig,x
noIncValue:

 lda #%00000100
 and pad2Buttons
 beq noDecValue
 ldx gloveConfigIndex
 dec gloveConfig,x
noDecValue:

    ; start (on release) flashes glove
 lda #%00010000
 and pad2Buttons
 bne noFlashGlove
 lda #%00010000
 and pad2Old
 beq noFlashGlove
 jsr flashGlove
noFlashGlove:

    ; A/B (on press) increase/decrease frame delay
    ; with Select held: increase/decrease poll bytes
 lda #%00100000
 and pad2Buttons
 bne changePollBytes

 lda frameDelay
 cmp #15
 beq noIncDelay
 lda #%10000000
 and pad2Buttons
 beq noIncDelay
 lda #%10000000
 and pad2Old
 bne noIncDelay
 inc frameDelay
noIncDelay:
 
 lda frameDelay
 cmp #2
 beq noDecDelay
 lda #%01000000
 and pad2Buttons
 beq noDecDelay
 lda #%01000000
 and pad2Old
 bne noDecDelay
 dec frameDelay
noDecDelay:
 jmp pad2done

changePollBytes:
 lda pollBytes
 cmp #17
 beq noIncPoll
 lda #%10000000
 and pad2Buttons
 beq noIncPoll
 lda #%10000000
 and pad2Old
 bne noIncPoll
 inc pollBytes
noIncPoll:
 
 lda pollBytes
 cmp #1
 beq noDecPoll
 lda #%01000000
 and pad2Buttons
 beq noDecPoll
 lda #%01000000
 and pad2Old
 bne noDecPoll
 dec pollBytes
noDecPoll:


pad2done:

    ; no actual need to distribute the glove poll across 2 frames.
    ; while the glove needs a few frames to be ready for a new poll,
    ; they will be strictly graphics and padding
 
 jsr waitNMI
 
    ; draw the first 8 glove bytes now that we have them fresh (we can only update ~9 of these lines per frame)
 lda #$20
 sta $2006
 sta temp0
 lda #$A1
 sta $2006 ; set PPU address
 sta temp1
 
 ldx #0 ; byte index
bitsUpdate2ByteLoop:
 ldy #8 ; bit counter
 lda gloveData,X
bitsUpdate2BitLoop:
 sta temp2
 ; rol a
 .db $2A ; stupid nesasm
 ; rol a
 .db $2A ; stupid nesasm
 and #%00000001
 sta $2007
 lda temp2
 asl A
 dey
 bne bitsUpdate2BitLoop
 
 lda temp1
 clc
 adc #$20
 sta temp1
 cmp #$01
 bne bitsUpdate2SkipAddrHi
 inc temp0
bitsUpdate2SkipAddrHi:
 lda temp0
 sta $2006
 lda temp1
 sta $2006
 inx
 cpx #8
 bne bitsUpdate2ByteLoop
 
 ; glove feedback sprite (when in canonical mode 1)
 lda #$FC
 sta $2003
 lda gloveY
 eor #$FF ; Y is up-positive, screen is down-positive, so...
 clc
 adc #1
 clc
 adc #122 ; native raw range now "-127" to "128" so add (127 - sprite-half-height - 1 (sprites are "delayed by 1 scanline"?))
 sta $2004
 lda gloveRot
 sta $2004
 lda #%00000000 ; vhb???cc - [v]ert flip, [h]oriz flip, [b]ehind bg, [c]olor palette
 sta temp0
 lda gloveFingers
 and #%00110000
 beq sprite_no_finger1
 inc temp0
sprite_no_finger1:
 lda gloveFingers
 and #%00001100
 beq sprite_no_finger2
 inc temp0
 inc temp0
sprite_no_finger2:
 lda temp0
 sta $2004
 lda gloveX
 clc
 adc #124 ; raw range is -128 to 127, so add (128 - sprite-half-width)
 sta $2004
 lda #0
 sta $2003 ; maybe I need to reset the sprite address too?  I shouldn't be getting mouse-trails on the NES...
 
 lda #0 ; proper way to end the update: zero the PPU address
 sta $2006
 sta $2006

    ; and in closing, yes, it really does need 4 full 60Hz frames
    ; to get its bits together for the next poll.  15fps updates ftw.
 ldx frameDelay
 dex
 dex
 stx waitdelay
 jsr waitloop

 jmp main_loop	; loop
 
title:
    ; Power Glove data viewer
    .db $50,$6F,$77,$65,$72,$20,$47,$6C,$6F,$76,$65,$20,$74,$65,$73,$74,$20,$74,$6F,$6F,$6C,$FF
copyright:
    ; (c) Sean Kelly / PsychSoftware
    .db $28,$63,$29,$20,$53,$65,$61,$6E,$20,$4B,$65,$6C,$6C,$79,$20,$2F,$20,$50,$73,$79,$63,$68,$53,$6F,$66,$74,$77,$61,$72,$65,$FF
date:
    ; April 2012
    .db $41,$70,$72,$69,$6C,$20,$02,$00,$01,$02,$FF
 
	.bank 1
	.org $E000


; generic footer
	.org $fffa
	.dw nmi		;//NMI
	.dw start	;//Reset
	.dw irq		;//IRQ

; mandatory chr data
	.bank 2
	.org $0000
	.incbin "fullascii_alt.chr"
	.incbin "fullascii_alt.chr"
	